#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['change_detection']

package_data = \
{'': ['*']}

install_requires = \
['pandas >= 0.24',
 'pandas-gbq >= 0.8.0',
 'numpy',
 'rpy2',
 'ebmdatalab >=0.0.20']

setup(name='change_detection',
      version='0.3.0',
      description='package for detecting change in time-series data',
      author='Alex Walker',
      author_email='alex.walker@phc.ox.ac.uk',
      url='https://github.com/ebmdatalab/change_detection',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
